@@ -698,13 +698,8 @@ void ccid_error(int log_level, int error, const char *file, int line,
 			text = var_text;
 			break;
 	}
-#ifdef USE_OS_LOG
-	(log_level);
-	os_log_error(OS_LOG_DEFAULT, "%s:%d:%s %s", file, line, function, text);
-#else
 	log_msg(log_level, "%s:%d:%s %s", file, line, function, text);
 #endif
-#endif
 
 } /* ccid_error */
 
