@@ -92,6 +92,11 @@ typedef struct
 	 */
 	int cardProtocol;
 
+	/*
+	 * Reader protocols
+	 */
+	int dwProtocols;
+
 	/*
 	 * bInterfaceProtocol (CCID, ICCD-A, ICCD-B)
 	 */
@@ -272,7 +277,7 @@ void ccid_error(int log_level, int error, const char *file, int line,
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 
 /* convert a 4 byte integer in USB format into an int */
-#define dw2i(a, x) (unsigned int)((((((a[x+3] << 8) + a[x+2]) << 8) + a[x+1]) << 8) + a[x])
+#define dw2i(a, x) (unsigned int)(((((((unsigned int)a[x+3] << 8) + (unsigned int)a[x+2]) << 8) + (unsigned int)a[x+1]) << 8) + (unsigned int)a[x])
 
 /* all the data rates specified by ISO 7816-3 Fi/Di tables */
 #define ISO_DATA_RATES 10753, 14337, 15625, 17204, \
