@@ -727,6 +727,7 @@ again:
 				usbDevice[reader_index].ccid.bVoltageSupport = device_descriptor[5];
 				usbDevice[reader_index].ccid.sIFD_serial_number = NULL;
 				usbDevice[reader_index].ccid.gemalto_firmware_features = NULL;
+				usbDevice[reader_index].ccid.dwProtocols = dw2i(device_descriptor, 6);
 #ifdef ENABLE_ZLP
 				usbDevice[reader_index].ccid.zlp = FALSE;
 #endif
@@ -908,7 +909,7 @@ read_again:
 	DEBUG_XXD(debug_header, buffer, *length);
 
 #define BSEQ_OFFSET 6
-	if ((*length >= BSEQ_OFFSET)
+	if ((*length >= BSEQ_OFFSET +1)
 		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
 	{
 		duplicate_frame++;
@@ -1451,7 +1452,8 @@ static void *Multi_PollingProc(void *p_ext)
 		if (NULL == transfer)
 		{
 			rv = LIBUSB_ERROR_NO_MEM;
-			DEBUG_COMM2("libusb_alloc_transfer err %d", rv);
+			DEBUG_COMM3("libusb_alloc_transfer err %d %s", rv,
+				libusb_error_name(rv));
 			break;
 		}
 
@@ -1467,7 +1469,8 @@ static void *Multi_PollingProc(void *p_ext)
 		if (rv)
 		{
 			libusb_free_transfer(transfer);
-			DEBUG_COMM2("libusb_submit_transfer err %d", rv);
+			DEBUG_COMM3("libusb_submit_transfer err %d %s", rv,
+				libusb_error_name(rv));
 			break;
 		}
 
@@ -1479,7 +1482,8 @@ static void *Multi_PollingProc(void *p_ext)
 			rv = libusb_handle_events_completed(ctx, &completed);
 			if (rv < 0)
 			{
-				DEBUG_COMM2("libusb_handle_events err %d", rv);
+				DEBUG_COMM3("libusb_handle_events err %d %s", rv,
+					libusb_error_name(rv));
 
 				if (rv == LIBUSB_ERROR_INTERRUPTED)
 					continue;
@@ -1579,9 +1583,10 @@ static void *Multi_PollingProc(void *p_ext)
 
 	if (rv < 0)
 	{
-		DEBUG_CRITICAL4("Multi_PollingProc (%d/%d): error %d",
+		DEBUG_CRITICAL5("Multi_PollingProc (%d/%d): error %d %s",
 			usbDevice[msExt->reader_index].bus_number,
-			usbDevice[msExt->reader_index].device_address, rv);
+			usbDevice[msExt->reader_index].device_address,
+			rv, libusb_error_name(rv));
 	}
 
 	/* Wake up the slot threads so they will exit as well */
